# Generated by Django 3.2.18 on 2024-05-06 16:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_P1_notification', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='template',
            old_name='template_body',
            new_name='message',
        ),
    ]



import logging
import os

import requests

from requests.auth import AuthBase

from openlxp_P1_notification_project import settings

logger = logging.getLogger('dict_config_logger')

headers = {'Content-Type': 'application/json'}


"""
    Functions set up to extract environment values for
    Platform One Postal Service (P1PS) API

"""


def get_P1PS_base_endpoint():
    """Extracts P1PS base endpoint"""

    P1PS_domain = settings.P1PS_BASE_URL

    if P1PS_domain:
        P1PS_endpoint = "https://" + P1PS_domain
        logger.info("P1PS endpoint value  is present and set")
    else:
        logger.error(P1PS_domain)
        logger.error("P1PS endpoint value is absent and not set")

    return P1PS_endpoint


def get_P1PS_team_token():
    """Extracts P1PS base endpoint"""
    team_token = settings.P1PS_AUTH_TOKEN

    if team_token:
        logger.info("Team Token value  is present and set")
    else:
        logger.error(team_token)
        logger.error("Team Token value is absent and not set")

    return team_token


def get_P1PS_team_ID():
    """Extracts P1PS base endpoint"""
    team_id = settings.P1PS_TEAM_ID

    if team_id:
        logger.info("Team ID value  is present and set")
    else:
        logger.error(team_id)
        logger.error("Team ID value is absent and not set")

    return team_id


"""
    Configuration set up for Platform One Postal Service (P1PS)
    API requests

"""


class TokenAuth(AuthBase):
    """Attaches HTTP Authentication Header to the given Request object."""

    def __call__(self, r, token_name='EMAIL_AUTH'):
        # modify and return the request

        r.headers[token_name] = get_P1PS_team_token()
        return r


def SetCookies():
    """Sets requests cookies jar with P1 authorization cookies"""
    logger.info("Setting cookies for email request P1ps")

    jar = requests.cookies.RequestsCookieJar()
    if os.environ.get('COOKIE_NAME') and os.environ.get('COOKIE_VALUE'):
        jar.set(os.environ.get('COOKIE_NAME'),
                os.environ.get('COOKIE_VALUE'),
                domain=settings.P1PS_BASE_URL, path='/')
    logger.info(jar)

    return jar



from datetime import datetime as dt
import json
import logging
import copy


from django.core.management.base import (BaseCommand, CommandParser,
                                         CommandError)

from openlxp_P1_notification.management.utils.p1ps_requests import (
    overall_health, send_email)
from openlxp_P1_notification.models import (email, recipient)

from openlxp_P1_notification.serializer import EmailSerializer

logger = logging.getLogger('dict_config_logger')


def trigger_health_check():
    """Command to trigger email health check"""
    overall_health()


def trigger_update(email_type):
    """Command to trigger email notification"""

    trigger_health_check()

    body_data = EmailSerializer(email_type).data

    recipient_list = body_data['recipients']
    now = dt.now()
    datetimenow = now.strftime("%d/%m/%Y %H:%M:%S")

    template_data = EmailSerializer(email_type).data

    for recipient_email in recipient_list:
        body_data = copy.deepcopy(template_data)

        recipient_obj = recipient.objects.get(
            email_address=recipient_email)

        body_data['recipients'] = [recipient_email]

        if 'name' in email_type.template_type.template_inputs:
            body_data['template_inputs']['name'] = (recipient_obj.first_name +
                                                    " " +
                                                    recipient_obj.last_name)
        if 'datetime' in email_type.template_type.template_inputs:
            body_data['template_inputs']['datetime'] = datetimenow

        body_data = json.dumps(body_data)

        send_email(body_data, str(email_type.template_type))


class Command(BaseCommand):
    """Django command to send an emails to the filer/personas, when the log
    warning/error occurred in the metadata EVTVL process."""

    def add_arguments(self, parser: CommandParser) -> None:
        # parser.add_argument('email_references', nargs="+", type=str)
        parser.add_argument('email_references', type=str)
        return super().add_arguments(parser)

    def handle(self, *args, **options):
        """Email log notification is sent to filer/personas when warning/error
        occurred in EVTVL process"""
        # for email_reference in options['email_references']:
        try:
            email_type = email.objects.get(
                reference=options['email_references'])
        except email.DoesNotExist:
            raise CommandError('Email Reference "%s" does not exist' %
                               options['email_references'])

        trigger_update(email_type)



from rest_framework.views import APIView
from rest_framework.response import Response

from openlxp_P1_notification.management.utils.p1ps_requests import (
    get_email_request)


class EmailRequestView(APIView):
    """Handles HTTP requests for Email request data from P1PS"""

    def get(self, request, request_id):
        """This method defines an API to fetch email request
        using request ID"""

        email_request_response = get_email_request(request_id)

        return Response(email_request_response.json(),
                        status=email_request_response.status_code)



from datetime import datetime as dt
import json
import logging
import copy


from django.core.management.base import (BaseCommand, CommandParser,
                                         CommandError)

from openlxp_P1_notification.management.utils.p1ps_requests import (
    overall_health, send_email)
from openlxp_P1_notification.models import (email)

from openlxp_P1_notification.serializer import EmailSerializer

logger = logging.getLogger('dict_config_logger')


def trigger_health_check():
    """Command to trigger email health check"""
    overall_health()


def trigger_update(email_type, recipient_list, owner,
                                   list_name, list_url):
    """Command to trigger email for list updates"""

    trigger_health_check()

    now = dt.now()
    datetimenow = now.strftime("%d/%m/%Y %H:%M:%S")

    template_data = EmailSerializer(email_type).data

    for recipient_id in recipient_list:
        body_data = copy.deepcopy(template_data)

        body_data['recipients'] = [recipient_id[0]]
        name = recipient_id[1]
        author = owner

        if 'name' in email_type.template_type.template_inputs:
            max_len = body_data['template_inputs']['name']['max_length']
            body_data['template_inputs']['name'] = name[:max_len]
        if 'datetime' in email_type.template_type.template_inputs:
            max_len = body_data['template_inputs']['datetime']['max_length']
            body_data['template_inputs']['datetime'] = datetimenow[:max_len]
        if 'list_name' in email_type.template_type.template_inputs:
            max_len = body_data['template_inputs']['list_name']['max_length']
            body_data['template_inputs']['list_name'] = list_name[:max_len]
        if 'list_url' in email_type.template_type.template_inputs:
            max_len = body_data['template_inputs']['list_url']['max_length']
            body_data['template_inputs']['list_url'] = list_url[:max_len]
        if 'author' in email_type.template_type.template_inputs:
            max_len = body_data['template_inputs']['author']['max_length']
            body_data['template_inputs']['author'] = author[:max_len]


        body_data = json.dumps(body_data)

        send_email(body_data, str(email_type.template_type))


class Command(BaseCommand):
    """Django command to send an emails to the filer/personas, when the log
    warning/error occurred in the metadata EVTVL process."""

    def add_arguments(self, parser: CommandParser) -> None:
        # parser.add_argument('email_references', nargs="+", type=str)
        parser.add_argument('--email', type=str)
        parser.add_argument('--recipient-list', metavar='N',
                            type=str, nargs='+', help='a list of strings')

        return super().add_arguments(parser)

    def handle(self, *args, **options):
        """Email log notification is sent to filer/personas when warning/error
        occurred in EVTVL process"""
        # for email_reference in options['email_references']:
        try:
            email_type = email.objects.get(
                reference=options['email'])
        except email.DoesNotExist:
            raise CommandError('Email Reference "%s" does not exist' %
                               options['email'])

        trigger_update(email_type,
                       [tuple(options['recipient_list'])],
                       "Owner",
                       "List_Name",
                       "List_url")




from django.core.management.base import BaseCommand

from openlxp_P1_notification.management.utils.p1ps_requests import (
    get_team_templates)


class Command(BaseCommand):
    """Django command to send an emails to the filer/personas, when the log
    warning/error occurred in the metadata EVTVL process."""

    def handle(self, *args, **options):
        """Load email templates from P1PS"""
        get_team_templates()



from openlxp_P1_notification.management.utils.p1ps_requests import (
    get_team_templates)


class TemplateMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        if request.path == "/admin/openlxp_P1_notification/template/":
            get_team_templates()

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response



import logging

from django.core.exceptions import ValidationError

import requests


from openlxp_P1_notification.management.utils.p1ps_configuration import (
    SetCookies, TokenAuth, get_P1PS_base_endpoint, get_P1PS_team_ID,
    get_P1PS_team_token)
from openlxp_P1_notification.serializer import TemplateSerializer


logger = logging.getLogger('dict_config_logger')

headers = {'Content-Type': 'application/json'}


"""
    Requests set up to verify using Platform One Postal Service (P1PS)
    API Specification (1.0.0)

"""


def SendResponse(response):
    """Function to return response and error catches"""

    if response.status_code in [400, 401, 404, 500]:
        logger.error(response.json())
        return False
    elif response.status_code in [200, 201]:
        try:
            logger.info(response.json())
        except requests.RequestException:
            logger.warning(requests.RequestException)
        return True


def get_team():
    """Request to verify user is a part team and check permissions"""
    base_endpoint = get_P1PS_base_endpoint()
    team_id = get_P1PS_team_ID()
    team_id_list = []
    response_success= False

    try:
        url = base_endpoint + "/api/teams"
        response = requests.get(url=url, headers=headers,
                                auth=TokenAuth(), cookies=SetCookies())
        response_success = SendResponse(response)
    except requests.exceptions.RequestException as e:
        logger.error(url)
        logger.error(e) 
    if response_success:
        for team in response.json()['data']:
            """iterating through different teams to find matching teams"""
            if 'team_id' in team:
                team_id_list += [team['team_id']]

        if team_id not in team_id_list:
            raise ValidationError("User does not have permission "
                                  "to use Team ID ")


def get_team_templates():
    """Request to get templates associated with team """
    base_endpoint = get_P1PS_base_endpoint()
    team_id = get_P1PS_team_ID()
    response_success = False

    try:
        url = base_endpoint + "/api/teams/" + team_id + "/templates"
        response = requests.get(url=url, headers=headers,
                                auth=TokenAuth(), cookies=SetCookies())
        response_success = SendResponse(response)
    except requests.exceptions.RequestException as e:
        logger.error(url)
        logger.error(e) 
    if response_success:
        for template in response.json()['data']:
            serializer = TemplateSerializer(data=template)

            if not serializer.is_valid():
                # If not received send error and bad request status
                logger.error(serializer.errors)
            else:
                # If received save record in templates
                logger.info(serializer)
                serializer.save()


"""
    Requests set up using Platform One Postal Service (P1PS)
    API Specification (1.0.0)

"""


def overall_health():
    """Request to perform P1PS health check """
    base_endpoint = get_P1PS_base_endpoint()
    get_P1PS_team_token()
    get_P1PS_team_ID()
    response_success = False

    try:
        url = base_endpoint + "/api/health"
        response = requests.get(url=url, headers=headers,
                                auth=TokenAuth(), cookies=SetCookies())
        response_success = SendResponse(response)
    except requests.exceptions.RequestException as e:
        logger.error(url)
        logger.error(e) 
    if response_success:
        get_team()


def send_email(body_data, template_type):
    """Request to send email via P1PS"""

    # Check user Permissions to use team

    base_endpoint = get_P1PS_base_endpoint()
    team_id = get_P1PS_team_ID()

    try:
        url = base_endpoint + "/api/teams/" + team_id + "/emails/" + template_type
        response = requests.post(url=url, headers=headers,
                                data=body_data, auth=TokenAuth(),
                                cookies=SetCookies())
    except requests.exceptions.RequestException as e:
        logger.error(url)
        logger.error(e) 
    SendResponse(response)


def get_email_request(request_id):
    """Retrieves an Email request based on the requested ID."""
    base_endpoint = get_P1PS_base_endpoint()
    team_id = get_P1PS_team_ID()

    try:
        url = base_endpoint + "/api/teams/" + team_id + "/emails/" + request_id
        response = requests.get(url=url, headers=headers,
                                auth=TokenAuth(), cookies=SetCookies())
    except requests.exceptions.RequestException as e:
        logger.error(url)
        logger.error(e) 
    return response



import logging
from rest_framework import serializers

from .models import (email, recipient, subject, template)

logger = logging.getLogger('dict_config_logger')


class EmailSerializer(serializers.ModelSerializer):
    """Serializer for model Email"""
    recipients = serializers.SlugRelatedField(
        many=True, slug_field='email_address',
        queryset=recipient.objects.all())
    subject = serializers.SlugRelatedField(slug_field='subject',
                                           queryset=subject.objects.all(), )
    template_inputs = serializers.SerializerMethodField('get_template_inputs')

    class Meta:
        model = email
        fields = ['recipients', 'subject', 'template_inputs']

    def get_template_inputs(self, email):
        template_inputs = email.template_type.template_inputs
        return template_inputs


class TemplateSerializer(serializers.ModelSerializer):
    """Serializer for model Template"""
    class Meta:
        model = template
        fields = ['template_type', 'message', 'template_inputs']

    def save(self):
        """Save function to create new templates """

        # # Assigning validated data as dictionary for updates in records
        validated_data = dict(
            list(self.validated_data.items())
        )

        # If value to update is present in metadata ledger
        if self.instance is None:

            self.instance = self.create(validated_data)
        else:
            logger.info("No new templates found for the Team")

        return self.instance



"""openlxp_P1_notification_project URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from . import views
from rest_framework.routers import DefaultRouter
from django.urls import path

router = DefaultRouter()

app_name = 'openlxp_P1_notification'
urlpatterns = [
    path('get-email-request/<str:request_id>',
         views.EmailRequestView.as_view(),
         name='email-request'),
]



"""
WSGI config for openlxp_P1_notification_project project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/howto/deployment/wsgi/
"""

import os

from django.core.wsgi import get_wsgi_application

from openlxp_P1_notification.management.utils.p1ps_requests import (
    get_team_templates)

os.environ.setdefault('DJANGO_SETTINGS_MODULE',
                      'openlxp_P1_notification_project.settings')
get_team_templates()


application = get_wsgi_application()



# Generated by Django 3.2.18 on 2024-05-15 18:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_P1_notification', '0002_rename_template_body_template_message'),
    ]

    operations = [
        migrations.RenameField(
            model_name='recipient',
            old_name='name',
            new_name='first_name',
        ),
        migrations.AddField(
            model_name='recipient',
            name='last_name',
            field=models.CharField(default='name', help_text='Enter recipient name', max_length=200),
            preserve_default=False,
        ),
    ]
